// prntview.cpp : implementation of the CPrntView class
//  view used to print a list of files in listed in associated CPrntDoc class
//  view itself remains hidden, used to support the printing process
//  Doug Overmyer 7/26/93

#include "stdafx.h"
#include "album.h"
    
#include "albuminc.h"    
#include "dibdoc.h" 
#include "mainfrm.h"
#include "prntdoc.h"
#include "prntview.h"    
#include "stsmtr.h"
#include "dibapi.h" 
#include <strstrea.h>    

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////  Message Map     ////////////////////
// CPrntView

IMPLEMENT_DYNCREATE(CPrntView, CView)

BEGIN_MESSAGE_MAP(CPrntView, CView)
	//{{AFX_MSG_MAP(CPrntView)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

////////////////////////  Creation/Destruction  ////////////////////////////
// CPrntView construction/destruction

CPrntView::CPrntView()
{
}

CPrntView::~CPrntView()
{
}

////////////////////////////  On Handlers      ///////////////////////////
void CPrntView::OnActivateView(BOOL bActivate, CView* pActivateView,
					CView* pDeactiveView)
{
	CView::OnActivateView(bActivate, pActivateView, pDeactiveView); 
	GetParentFrame()->ShowWindow(SW_HIDE);  
	ShowWindow(SW_HIDE);
}

void CPrntView::OnDraw(CDC* pDC)
{

}


void CPrntView::OnInitialUpdate()
{                            
	//this view is never displayed - it is used to run the printing process
	CView::OnInitialUpdate();  
	GetParentFrame()->ShowWindow(SW_HIDE);  
	ShowWindow(SW_HIDE);	
}

void CPrntView::OnPrepareDC(CDC* pDC,CPrintInfo* pInfo)
{ 
	CView::OnPrepareDC(pDC,pInfo);                   

	if (!pInfo->m_bContinuePrinting)
		pInfo->m_bContinuePrinting = m_bIsMoreToPrint; 
}

BOOL CPrntView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
} 

void CPrntView::OnPrint(CDC* pDC,CPrintInfo* pInfo)
{    
	CStringList* 		pfileList; 
	POSITION		fpos;
	CString  		file;
	CRect				tr;
	WORD				i = 0,w=0,h=0,r=0,c=0,pgStart=0,pgEnd=0;  
	SState*			st= GetDocument()->GetStateInfo();
	CPoint			grid(st->m_across,st->m_down); 
	// get StatusBar object for updates
	CStsMtr * pStatusBar = (CStsMtr *)
				LOWORD(AfxGetApp()->m_pMainWnd->SendMessage(UM_GETSTATUSBAR,0,0)); 
	// get application pointer for calls 
	CAlbumApp* pApp = ((CAlbumApp*)AfxGetApp());                
	//get State structure for state info
	SState __far* lpSt = (SState __far* )
			AfxGetApp()->m_pMainWnd->SendMessage(UM_GETSTATEINFO,0,0);     
	// get device resolution (logical pixels per inch)
	int cxInch = pDC->GetDeviceCaps(LOGPIXELSX);
	int cyInch = pDC->GetDeviceCaps(LOGPIXELSY);    
	// convert margins from inches to logical units, store in a CRect
	CRect mRect(int(lpSt->m_lMargin*cxInch),int(lpSt->m_tMargin*cyInch),
			int(lpSt->m_rMargin*cxInch),int(lpSt->m_bMargin*cyInch)); 
     
  // substract margins from drawing rectangle
	pInfo->m_rectDraw.left +=mRect.left;   
	pInfo->m_rectDraw.top +=mRect.top;
	pInfo->m_rectDraw.right -=mRect.right;
	pInfo->m_rectDraw.bottom -=mRect.bottom;
	  
	// get size of printer page (in pixels)
	int cxPage = pDC->GetDeviceCaps(HORZRES);
	int cyPage = pDC->GetDeviceCaps(VERTRES);
	// take a rest
	pApp->Take5();
	//get the list of files to be printed
	pfileList = GetDocument()->GetFileList(); 
	//get first file name
	fpos=pfileList->GetHeadPosition(); 
	file = pfileList->GetNext(fpos); 
	//print a header using first filename 
	PrintHeader(pDC,pInfo,&file);	   
	                                     
  //get width,height of printable are from m_rectDraw
	int cxPrint=pInfo->m_rectDraw.Width();
	int cyPrint=pInfo->m_rectDraw.Height();            
	//get width & height allowed for each thumbnail w/ 10% margin for spacing
	w=::MulDiv((pInfo->m_rectDraw.Width()/grid.x),9,10);    //10% margin
	h=::MulDiv((pInfo->m_rectDraw.Height()/grid.y),9,10);    //10% margin
	                                              
	//calc beginning and ending number of item to be printed on this page                                              
	pgStart=(pInfo->m_nCurPage-1)*grid.x*grid.y;
	pgEnd = pgStart+(grid.x*grid.y)-1;
	m_bIsMoreToPrint=FALSE;  
	//activate status bar meter
  pStatusBar->SetPct(1); 
  
  UINT uCnt = pfileList->GetCount();
  
  //iterate over the whole file list
	fpos=pfileList->GetHeadPosition();	
	while (fpos != NULL)
		{
		file = pfileList->GetNext(fpos);  
		i++;                         
		//skip if on previous page
		if (i < pgStart+1)	continue;     
		//compute a target rect each graphic will draw itself into
		tr.SetRectEmpty();
		tr.left=pInfo->m_rectDraw.left+c*(cxPrint/grid.x);
		tr.right=tr.left + w; 
		tr.top= pInfo->m_rectDraw.top+ r*(cyPrint/grid.y);
		tr.bottom=tr.top + h;                                     
		//load the graphic
		GetDocument()->InitDIB(file);  
		//print a caption
		PrintCaption(pDC,pInfo,&tr,&file);  
		//print the graphic
		PrintThumb(pDC, pInfo,tr);
		// update the meter   
		pStatusBar->SetPct(i * 100 / uCnt); 
		//take a break  
		pApp->Take5();		
		c++;          
		//where are we on the grid?
		if (!( c < WORD(grid.x)))
			{
			c=0;
			r++;
			}  
		if (i > pgEnd)
			{
			m_bIsMoreToPrint = TRUE;
			break;
			}
		}                          
	// close down meter if done
	if (!m_bIsMoreToPrint)
  	pStatusBar->SetPct(0); 
}
                
//////////////////////////   Operations  /////////////////////////////////
void CPrntView::PrintThumb(CDC* pDC,CPrintInfo* pInfo, CRect tr) 
{
	CPrntDoc* 			pDoc = GetDocument(); 
	CMainFrame* 	pMF = GetMainFrame();	    
	CMDIChildWnd* pDocFrame = GetDocFrame();
	HDIB 					hDIB = pDoc->GetHDIB();
	WORD					cx,cy,maxxy; 
	
	if (hDIB != NULL)
		{
		LPSTR lpDIB = (LPSTR) ::GlobalLock((HGLOBAL) hDIB);
		int cxDIB = (int) ::DIBWidth(lpDIB);         // Size of DIB - x
		int cyDIB = (int) ::DIBHeight(lpDIB);        // Size of DIB - y
		::GlobalUnlock((HGLOBAL) hDIB);
		CRect rcImg;
		rcImg.top = rcImg.left = 0;
		rcImg.right = cxDIB;
		rcImg.bottom = cyDIB;
		CRect rcDest; 
		maxxy=max(cxDIB,cyDIB);   
		cx=tr.Width();
		cy=tr.Height();
		if (cx < cy)                    //square the target area                  
			tr.bottom = tr.top + cx;
		else
			tr.right = tr.left + cy; 
		cx=tr.Width();
		cy=tr.Height();                              
		//scale the target to the DIB dimensions
		tr.right=tr.left+ ::MulDiv(cx,cxDIB,maxxy);  
		tr.bottom=tr.top+ ::MulDiv(cy,cyDIB,maxxy);
		
		::PaintDIB(pDC->m_hDC, &tr, pDoc->GetHDIB(),
					&rcImg, pDoc->GetDocPalette());
		}
}

void CPrntView::PrintCaption(CDC* pDC,CPrintInfo* pInfo, CRect* pTr,CString* pfile)
{
  CString caption ;                         
  // find the trailing backslash
  int Indx = pfile->ReverseFind('\\');  
  if (Indx == -1) return;
  // extract the simple filename
  caption=pfile->Mid(Indx+1);
  //get text extent
	CSize dtSize = pDC->GetOutputTextExtent(caption,caption.GetLength()); 
	// make copy of the 'target rectangle'
	CRect dtRect(*pTr);                   
	// adjust target rect to print at bottom
	dtRect.top = dtRect.bottom - dtSize.cy;
	//draw in the rect
	pDC->DrawText(caption,-1,dtRect,DT_LEFT);	  
	// adjust pTr for text
  pTr->SubtractRect(pTr,&dtRect);
}

void CPrntView::PrintHeader(CDC* pDC,CPrintInfo* pInfo,CString* pfile)
{   	
	char buf[80];        
	ostrstream msg(buf,sizeof(buf));
  CString caption,caption1 ;
  
  int Indx = pfile->ReverseFind('\\');  
  if (Indx == -1) return;       
  // get the pathname
  caption=pfile->Mid(0,Indx); 
  msg<<"Dir:"<<caption<<"   page:"<<pInfo->m_nCurPage <<ends;  
  
  caption1 = buf;
  //get text rectangle for caption
	CSize dtSize = pDC->GetOutputTextExtent(caption1,caption1.GetLength()); 
	//make copy of printing area (we are at top of page!)
	CRect dtRect(pInfo->m_rectDraw);        
	// size to text extents
	dtRect.bottom = dtRect.top + dtSize.cy;
	//draw the text
	pDC->DrawText(caption1,-1,dtRect,DT_LEFT);	  
	//adjust page area as needed
  pInfo->m_rectDraw.top += dtSize.cy * 2;    //adjust the drawing rect
}